var React = global.React || require('react');
var action = require('./action.js');
var EventEmitter = require('eventemitter2').EventEmitter2 || require('eventemitter2');
var safeDeepClone = require('./safeDeepClone.js');
var RenderMixin = require('./RenderMixin.js');

var flux = {};

function mergeStore (mixins, source) {

  source.actions = source.actions || [];
  source.exports = source.exports || {};

  if (mixins && Array.isArray(mixins)) {

    // Merge mixins and state
    mixins.forEach(function (mixin) {
      Object.keys(mixin).forEach(function (key) {

        switch(key) {
          case 'mixins':
            return mergeStore(mixin.mixins, mixin);
          case 'actions':
            source.actions = source.actions.concat(mixin.actions);
            break;
          case 'exports':
            Object.keys(mixin.exports).forEach(function (key) {
              source.exports[key] = mixin.exports[key];
            });
            break;
          default:
            if (source[key]) {
              throw new Error('The property: ' + key + ', already exists. Can not merge mixin with keys: ' + Object.keys(mixin).join(', '));
            }
            source[key] = mixin[key];
        }

      });
    });

  }

  var exports = new EventEmitter({
    wildcard: true
  });

  source.emitChange = function () {
    setTimeout(function () { // Async to avoid running within render of component
      exports.emit('change');
    }, 0);
  };

  source.emit = function () {
    var args = arguments;
    setTimeout(function () { // Async to avoid running within render of component
      exports.emit.apply(exports, args);
    }, 0);
  };

  exports.addChangeListener = function (callback) {
    exports.on('change', callback);
  };

  exports.removeChangeListener = function (callback) {
    exports.removeListener('change', callback);
  };

  // Register actions
  source.actions.forEach(function (action) {
    if (!action || !action.handlerName) {
      throw new Error('This is not an action ' + action);
    }
    if (!source[action.handlerName]) {
      throw new Error('There is no handler for action: ' + action);
    }
    action.on('trigger', source[action.handlerName].bind(source));
  });

  // Register exports
  Object.keys(source.exports).forEach(function (key) {
    exports[key] = function () {
      return safeDeepClone('[Circular]', [], source.exports[key].apply(source, arguments));
    };
  });

  return exports;

};

flux.debug = function () {
  global.React = React;
};

flux.createActions = function () {
  return action.apply(null, arguments);
};

flux.createStore = function (definition) {
  return mergeStore(definition.mixins, definition);
};

flux.RenderMixin = RenderMixin;

// If running in global mode, expose $$
if (!global.exports && !global.module && (!global.define || !global.define.amd)) {
  global.flux = flux;
}

module.exports = flux;
